const express = require('express');
const path = require('path');
const { createProxyMiddleware } = require('http-proxy-middleware');
const https = require('https');
const fs = require('fs');

const options = {
    key: fs.readFileSync('./keys/server.key'),
    cert: fs.readFileSync('./keys/server.crt')
};

const app = express();
const httpsServer = https.createServer(options, app);
const config = require('./tums-player-serve/config.json');

const proxyApis = ['/tums', '/vms'];

proxyApis.forEach(api => {
    app.use(
        api,
        createProxyMiddleware({
            target: config.proxyServer,
            changeOrigin: true,
            secure: false,
            headers: {
                Referer: config.proxyServer
            }
        })
    );
});

app.use('/', express.static(__dirname + '/tums-player-serve/'));

app.get('/', (_, res) => {
    res.sendFile(path.join(__dirname + '/tums-player-serve/', 'index.html'));
});

app.listen(config.port, () => {
    console.log('http server running on: ' + `http://${config.host || 'localhost'}:${config.port}/`);
});

const httpsPort = parseInt(config.port) + 1;
httpsServer.listen(httpsPort, () => {
    console.log('https server running on: ' + `https://${config.host || 'localhost'}:${httpsPort}/`);
});
